var View, addId, addNameToData, attrEscape, dataValue, extend, extractPlaceholder, htmlEscape, htmlParser, modelPath, modelText, parse, parseAttr, parseElement, parsePlaceholder, parseString, reduceStack, renderer, trim, trimInner, _ref;
htmlParser = require('./htmlParser');
_ref = require('./parser'), modelPath = _ref.modelPath, parsePlaceholder = _ref.parsePlaceholder, parseElement = _ref.parseElement, parseAttr = _ref.parseAttr;
View = module.exports = function() {
  var self;
  self = this;
  this._views = {};
  this._paths = {};
  this._loadFuncs = '';
  this._idCount = 0;
  this._uniqueId = function() {
    return '$' + (self._idCount++).toString(36);
  };
};
View.prototype = {
  get: function(viewName, ctx, parentCtx, path, triggerPath) {
    var i, indicies, item, obj, out, parentView, paths, re, type, view, _len, _ref2;
    if (!(view = this._views[viewName])) {
      if (~(i = viewName.indexOf('$'))) {
        parentView = viewName.substr(1, i - 1);
        if (!this._views[parentView]) {
          throw "Can't find view: " + parentView;
        }
        this.get(parentView, {
          $triggerPath: triggerPath
        });
        return this.get(viewName, ctx, parentCtx, null, triggerPath);
      }
      return '';
    }
    paths = parentCtx && parentCtx.$paths;
    if (path) {
      paths = this._paths[viewName] = paths ? [path].concat(paths) : [path];
    } else {
      this._paths[viewName] = paths || (paths = this._paths[viewName]);
    }
    type = viewName.charAt(0);
    if (ctx && ctx.splice && ctx.slice) {
      if (paths) {
        paths[0] += '.$#';
        parentCtx = Object.create(parentCtx);
        parentCtx.$paths = paths;
      }
      if (ctx.length) {
        if (type === '^') {
          return '';
        }
        out = '';
        indicies = parentCtx.$i || [];
        for (i = 0, _len = ctx.length; i < _len; i++) {
          item = ctx[i];
          obj = extend(parentCtx, item);
          obj.$i = [i].concat(indicies);
          out += view(obj);
        }
        return out;
      } else {
        if (type !== '^') {
          return '';
        }
      }
    } else {
      if ((type === '^' && ctx) || (type === '#' && !ctx)) {
        return '';
      }
    }
    ctx = extend(parentCtx, ctx);
    if ((ctx.$paths = paths) && (triggerPath || (triggerPath = ctx.$triggerPath))) {
      re = RegExp(paths[0].replace(/\$#|\./g, function(match) {
        if (match === '.') {
          return '\\.';
        } else {
          return '(\\d+)';
        }
      }));
      ctx.$i = (_ref2 = re.exec(triggerPath)) != null ? _ref2.slice(1).reverse() : void 0;
    }
    return view(ctx);
  },
  preLoad: function(fn) {
    return this._loadFuncs += "(" + fn + ")();";
  },
  make: function(name, template, data) {
    var fn, render, self;
    if (data == null) {
      data = {};
    }
    self = this;
    render = function(ctx) {
      if (typeof template === 'string' && !~template.indexOf('{{')) {
        template = trim(template);
        render = function() {
          return template;
        };
      } else {
        render = parse(self, name, template, data);
      }
      return render(ctx);
    };
    fn = function(ctx) {
      return render(extend(data, ctx));
    };
    return this._register(name, fn, data.Before, data.After);
  },
  _register: function(name, fn, before, after) {
    return this._views[name] = before ? after ? function(ctx) {
      var rendered;
      before();
      setTimeout(after, 0);
      return rendered = fn(ctx);
    } : function(ctx) {
      before();
      return fn(ctx);
    } : after ? function(ctx) {
      var rendered;
      setTimeout(after, 0);
      return rendered = fn(ctx);
    } : fn;
  }
};
extend = function(parent, obj) {
  var key, out;
  if (typeof parent !== 'object') {
    if (typeof obj === 'object') {
      return obj;
    } else {
      return {};
    }
  }
  out = Object.create(parent);
  if (!obj) {
    return out;
  }
  for (key in obj) {
    out[key] = obj[key];
  }
  return out;
};
addId = function(attrs, uniqueId) {
  if (attrs.id == null) {
    return attrs.id = function() {
      return attrs._id = uniqueId();
    };
  }
};
View.htmlEscape = htmlEscape = function(s) {
  if (s == null) {
    return '';
  } else {
    return s.toString().replace(/&(?!\s)|</g, function(s) {
      if (s === '&') {
        return '&amp;';
      } else {
        return '&lt;';
      }
    });
  }
};
View.attrEscape = attrEscape = function(s) {
  if (s == null || s === '') {
    return '""';
  }
  s = s.toString().replace(/&(?!\s)|"/g, function(s) {
    if (s === '&') {
      return '&amp;';
    } else {
      return '&quot;';
    }
  });
  if (/[ =<>']/.test(s)) {
    return '"' + s + '"';
  } else {
    return s;
  }
};
trim = function(s) {
  if (s) {
    return s.replace(/(?:^|\n)\s*/g, '');
  } else {
    return '';
  }
};
trimInner = function(s) {
  if (s) {
    return s.replace(/\n\s*/g, '');
  } else {
    return '';
  }
};
extractPlaceholder = function(text) {
  var content, match;
  match = /^([^\{]*)(\{{2,3})([^\}]+)\}{2,3}([\s\S]*)/.exec(text);
  if (!match) {
    return;
  }
  content = /^([#^//]?) *(~?) *([^ >]*)(?: *> *(.*))?/.exec(match[3]);
  return {
    pre: trimInner(match[1]),
    escaped: match[2] === '{{',
    type: content[1],
    literal: content[2],
    name: content[3],
    partial: content[4],
    post: trimInner(match[4])
  };
};
addNameToData = function(data, name) {
  if (name && !(name in data)) {
    return data[name] = {
      model: name
    };
  }
};
dataValue = function(data, name, model) {
  var path, value;
  if (path = modelPath(data, name)) {
    if ((value = model.get(path)) != null) {
      return value;
    } else {
      return model[path];
    }
  } else {
    return data[name];
  }
};
modelText = function(view, name, escape) {
  return function(data, model) {
    var text;
    text = dataValue(data, name, model);
    text = text != null ? text.toString() : '';
    if (escape) {
      text = escape(text);
    }
    return text;
  };
};
reduceStack = function(stack) {
  var bool, html, i, item, key, pushValue, value, _i, _len, _ref2;
  html = [''];
  i = 0;
  for (_i = 0, _len = stack.length; _i < _len; _i++) {
    item = stack[_i];
    pushValue = function(value, isAttr) {
      if (value && value.call) {
        return i = html.push(value, '') - 1;
      } else {
        return html[i] += isAttr ? attrEscape(value) : value;
      }
    };
    switch (item[0]) {
      case 'start':
        html[i] += '<' + item[1];
        _ref2 = item[2];
        for (key in _ref2) {
          value = _ref2[key];
          if (value != null) {
            if (bool = value.bool) {
              pushValue(bool);
              continue;
            }
            html[i] += ' ' + key + '=';
            pushValue(value, true);
          } else {
            html[i] += ' ' + key;
          }
        }
        html[i] += '>';
        break;
      case 'chars':
        pushValue(item[1]);
        break;
      case 'end':
        html[i] += '</' + item[1] + '>';
    }
  }
  return html;
};
renderer = function(view, items, events) {
  return function(data) {
    var domEvents, event, html, item, model, modelEvents, _i, _len;
    model = view.model;
    modelEvents = model.__events;
    domEvents = view.dom.events;
    html = ((function() {
      var _i, _len, _results;
      _results = [];
      for (_i = 0, _len = items.length; _i < _len; _i++) {
        item = items[_i];
        _results.push((item.call ? item(data, model) : item));
      }
      return _results;
    })()).join('');
    for (_i = 0, _len = events.length; _i < _len; _i++) {
      event = events[_i];
      event(data, modelEvents, domEvents);
    }
    return html;
  };
};
parse = function(view, viewName, template, data) {
  var block, chars, events, partialCount, partialName, popped, queue, queues, stack, uniqueId, _fn, _i, _len;
  if (viewName === 'Title') {
    return parseString(view, viewName, template, data);
  }
  queues = [
    {
      stack: stack = [],
      events: events = []
    }
  ];
  popped = [];
  block = null;
  uniqueId = view._uniqueId;
  partialCount = 0;
  partialName = function() {
    return viewName + '$' + partialCount++;
  };
  htmlParser.parse(template, {
    start: function(tag, tagName, attrs) {
      var attr, out, parser, value, _fn;
      if (parser = parseElement[tagName]) {
        out = parser(events, attrs) || {};
        if (out.addId) {
          addId(attrs, uniqueId);
        }
      }
      _fn = function(attr) {
        var anyOut, anyParser, args, bool, elOut, elParser, invert, literal, match, method, name, partial, post, pre, _ref2;
        if (match = extractPlaceholder(value)) {
          pre = match.pre, post = match.post, name = match.name, partial = match.partial, literal = match.literal;
          addNameToData(data, name);
          invert = /^\s*!\s*$/.test(pre);
          if ((pre && !invert) || post || partial) {
            console.log('TODO');
          }
          if (parser = parsePlaceholder[attr]) {
            if (anyParser = parser['*']) {
              anyOut = anyParser(events, attrs, name, invert);
            }
            if (elParser = parser[tagName]) {
              elOut = elParser(events, attrs, name, invert);
            }
          }
          anyOut || (anyOut = {});
          elOut || (elOut = {});
          method = elOut.method || anyOut.method || 'attr';
          bool = elOut.bool || anyOut.bool;
          if (((_ref2 = data[name]) != null ? _ref2.model : void 0) && !literal) {
            addId(attrs, uniqueId);
            events.push(function(data, modelEvents) {
              var args, path;
              args = [attrs._id || attrs.id, method, attr];
              if (invert) {
                args.push('$inv');
              }
              if (path = modelPath(data, name)) {
                return modelEvents.bind(path, args);
              }
            });
          }
          attrs[attr] = bool ? {
            bool: function(data, model) {
              if (!dataValue(data, name, model) !== !invert) {
                return ' ' + attr;
              } else {
                return '';
              }
            }
          } : modelText(view, name, attrEscape);
        }
        if (!(parser = parseAttr[attr])) {
          return;
        }
        args = value.replace(/\s/g, '').split(':');
        args.unshift(events, attrs);
        addId(attrs, uniqueId);
        if (anyParser = parser['*']) {
          anyOut = anyParser.apply(null, args);
        }
        if (elParser = parser[tagName]) {
          elOut = elParser.apply(null, args);
        }
        anyOut || (anyOut = {});
        elOut || (elOut = {});
        if (elOut.addId || anyOut.addId) {
          return addId(attrs, uniqueId);
        }
      };
      for (attr in attrs) {
        value = attrs[attr];
        _fn(attr);
      }
      return stack.push(['start', tagName, attrs]);
    },
    chars: chars = function(text) {
      var addEvent, attrs, autoClosed, datum, endBlock, escaped, i, last, lastAutoClosed, lastPartial, literal, match, name, partial, post, pre, startBlock, type, wrap, _ref2;
      if (!(match = extractPlaceholder(text))) {
        if (text = trimInner(text)) {
          stack.push(['chars', text]);
        }
        return;
      }
      pre = match.pre, post = match.post, name = match.name, escaped = match.escaped, type = match.type, partial = match.partial, literal = match.literal;
      addNameToData(data, name);
      if (pre) {
        stack.push(['chars', pre]);
      }
      if (block && ((endBlock = type === '/') || (autoClosed = type === '^' && (!name || name === block.name) && block.type === '#'))) {
        name = block.name, partial = block.partial, literal = block.literal, lastPartial = block.lastPartial, lastAutoClosed = block.lastAutoClosed;
        popped.push(queues.pop());
        _ref2 = queues[queues.length - 1], stack = _ref2.stack, events = _ref2.events, block = _ref2.block;
      }
      if (startBlock = type === '#' || type === '^') {
        lastPartial = partial;
        partial = type + partialName();
        block = {
          type: type,
          name: name,
          partial: partial,
          literal: literal,
          lastPartial: lastPartial,
          lastAutoClosed: autoClosed
        };
      }
      text = !partial ? '' : function(data, model) {
        return view.get(partial, dataValue(data, name || '.', model), data, modelPath(data, name, true));
      };
      if ((datum = data[name]) != null) {
        if (datum.model && !startBlock) {
          if (!literal) {
            i = stack.length - (endBlock ? (lastAutoClosed ? 3 : 2) : 1);
            last = stack[i];
            if (wrap = pre || (post && !/^\{{2,3}[\/^]\}{2,3}/.test(post)) || !(last && last[0] === 'start')) {
              last = ['start', 'ins', {}];
              if (endBlock) {
                stack.splice(i + 1, 0, last);
              } else {
                stack.push(last);
              }
            }
            attrs = last[2];
            addId(attrs, uniqueId);
            addEvent = function(partial, append) {
              return events.push(function(data, modelEvents) {
                var domMethod, params, path;
                if (!(path = modelPath(data, name))) {
                  return;
                }
                if (partial) {
                  escaped = false;
                }
                domMethod = append ? 'appendHtml' : 'html';
                params = [attrs._id || attrs.id, domMethod, +escaped];
                if (partial) {
                  params[3] = partial;
                }
                return modelEvents.bind(path, params);
              });
            };
            addEvent(partial);
            if (lastAutoClosed) {
              addEvent(lastPartial, true);
            }
          }
          text = text || modelText(view, name, escaped && htmlEscape);
        } else {
          text = text || (escaped ? htmlEscape(datum) : datum.toString());
        }
      }
      if (text && !endBlock) {
        stack.push(['chars', text]);
      }
      if (wrap) {
        stack.push(['end', 'ins']);
      }
      if (startBlock) {
        queues.push({
          stack: stack = [],
          events: events = [],
          viewName: partial,
          block: block
        });
      }
      if (post) {
        return chars(post);
      }
    },
    end: function(tag, tagName) {
      return stack.push(['end', tagName]);
    }
  });
  _fn = function(queue) {
    var render;
    render = renderer(view, reduceStack(queue.stack), queue.events);
    return view._register(queue.viewName, function(ctx) {
      return render(extend(data, ctx));
    });
  };
  for (_i = 0, _len = popped.length; _i < _len; _i++) {
    queue = popped[_i];
    _fn(queue);
  }
  return renderer(view, reduceStack(stack), events);
};
parseString = function(view, viewName, template, data) {
  var events, items, name, params, post, pre, _ref2;
  items = [];
  events = [];
  post = template;
  while (post) {
    _ref2 = extractPlaceholder(post), name = _ref2.name, pre = _ref2.pre, post = _ref2.post;
    addNameToData(data, name);
    if (pre) {
      items.push(pre);
    }
    items.push(modelText(view, name));
    if (viewName === 'Title') {
      params = ['$doc', 'prop', 'title', 'Title'];
    }
    (function(name) {
      if (params) {
        return events.push(function(data, modelEvents) {
          var path;
          if (!(path = modelPath(data, name))) {
            return;
          }
          return modelEvents.bind(path, params.slice());
        });
      }
    })(name);
  }
  return renderer(view, items, events);
};